/**
 * 
 */
package edu.wisc.uwss.local;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import edu.wisc.uwss.UWUserDetails;
import edu.wisc.uwss.UWUserDetailsImpl;

/**
 * Interface providing a mechanism to bind a row from the local users properties
 * file to a {@link UWUserDetails} instance.
 * 
 * @author Nicholas Blair
 */
public interface LocalUserDetailsAttributesMapper {

  /**
   * Map a list of attributes to a {@link UWUserDetails}.
   * 
   * @param username the value to use from {@link UWUserDetails#getUsername()} 
   * @param row the values from a row in the properties file, in order
   * @return a {@link UWUserDetails} instance (never null)
   * @throws IllegalArgumentException if the row is malformed
   */
  UWUserDetails mapUser(String username, String [] row);

  /**
   * Default {@link LocalUserDetailsAttributesMapper} implementation.
   * 
   * @author Nicholas Blair
   */
  public static class Default implements LocalUserDetailsAttributesMapper {

    /**
     * {@inheritDoc}
     * 
     * The format of the properties is as follows (key=value):
     * <pre>
     * username=password,fullName,firstName,lastName,emailAddress[,membership1,[membership2...]]
     * </pre>
     * 
     * Membership can be 1 or more UDDS values, comma separated.
     */
    @Override
    public UWUserDetails mapUser(String username, String[] values) {
      if(values.length > 6) {
        String[] uddsValues = Arrays.copyOfRange(values, 6, values.length);
        List<String> membership = new ArrayList<>();
        for(String v : uddsValues) {
          membership.add(v); 
        }
        // pvi, username, password, email,
        return new UWUserDetailsImpl(values[1], username, values[0], values[2], values[5], membership)
                .setFirstName(values[3])
                .setLastName(values[4]);
      } else {
        // String pvi, String username, String password, String fullName, String emailAddress
        return new UWUserDetailsImpl(values[1], username, values[0], values[2], values[5])
                .setFirstName(values[3])
                .setLastName(values[4]);
      }
    }
    
  }
}
